#ifndef MESH_VIEWER_H
#define MESH_VIEWER_H

#define GL_GLEXT_PROTOTYPES
#include "my_mesh.h"
#include <QMatrix4x4>
#include <QColor>
#include <QDebug>
#include <QOpenGLWidget>
#include <QOpenGLShaderProgram>
#include <QOpenGLFunctions>
#include <QMouseEvent>


#define WIREFRAME_COLOR 0, 0, 0
#define FOV 70


using namespace OpenMesh;


struct OpenGLMesh {
	GLuint vao;
	GLuint vbo;
	size_t nverts;
	QMatrix4x4 mat;
	QColor col;
};


class MeshViewer : public QOpenGLWidget {
    Q_OBJECT

public:
    MeshViewer(QWidget *parent=nullptr);
    void initializeGL() override;
    void resizeGL(int w, int h) override;
    void paintGL() override;
	OpenGLMesh *addOpenGLMesh(size_t nverts, const GLfloat *verts, QMatrix4x4 mat=QMatrix4x4(), QColor col={153, 153, 153});
	OpenGLMesh *addOpenGLMeshFromOpenMesh(MyMesh* _mesh, QMatrix4x4 mat=QMatrix4x4(), QColor col={153, 153, 153});
	void removeOpenGLMesh(OpenGLMesh *mesh);
	void setOpenGLMeshMatrix(OpenGLMesh *mv, QMatrix4x4 mat);

private:
	QList<OpenGLMesh> meshes;
	GLint pos_attr, col_attr, proj_attr, view_attr, model_attr;
	GLint wf_col_attr, wireframe_attr, alpha_attr;
	QMatrix4x4 proj;
	QMatrix4x4 rot, rot_start;
	GLfloat cam_dist = 1;
	QPoint mouse_pos;

protected:
    virtual void mousePressEvent(QMouseEvent *e);
    virtual void mouseReleaseEvent(QMouseEvent *e);
    virtual void mouseMoveEvent(QMouseEvent *e);
    virtual void wheelEvent(QWheelEvent *e);

signals:
	void initialized();
};


#endif
